
package com.kevinresol.react_native_default_preference;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.util.Log;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMapKeySetIterator;
import com.facebook.react.bridge.ReadableType;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class RNDefaultPreferenceModule extends ReactContextBaseJavaModule {
	private static final String TAG = "DEFAULT_PREFERENCE";

	private String preferencesName;

	private final ReactApplicationContext reactContext;

	public RNDefaultPreferenceModule(ReactApplicationContext reactContext) {
		super(reactContext);
		this.reactContext = reactContext;
		this.preferencesName = reactContext.getPackageName() + "_preferences";
	}

	@Override
	public String getName() {
		return "RNDefaultPreference";
	}

	@ReactMethod
	public void setMultiple(ReadableMap data, Promise promise) {
		putMultiple(data);
		promise.resolve(null);
	}

	@ReactMethod
	public void getMultiple(ReadableArray keys, Promise promise) {
		promise.resolve(getMultiple(keys.toArrayList()));
	}

	@ReactMethod
	public void getAll(Promise promise) {
		promise.resolve(getMultiple(null));
	}

	@ReactMethod
	public void clearMultiple(ReadableArray keys, Promise promise) {
		clearMultiple(keys.toArrayList());
		promise.resolve(null);
	}

	@ReactMethod
	public void clearAll(Promise promise) {
		clearMultiple(null);
		promise.resolve(null);
	}

	@ReactMethod
	public void setName(String preferencesName, Promise promise) {
		this.preferencesName = preferencesName;
		promise.resolve(null);
	}

	@ReactMethod
	public void getName(Promise promise) {
		promise.resolve(preferencesName != null ? preferencesName : "");
	}

	private void putMultiple(ReadableMap data) {
		try {
			if (data != null) {
				SharedPreferences.Editor editor = getEditor();
				ReadableMapKeySetIterator iter = data.keySetIterator();
				while (iter.hasNextKey()) {
					String key = iter.nextKey();
					if (data.getType(key) == ReadableType.String) {
						editor.putString(key, data.getString(key));
					} else if (data.getType(key) == ReadableType.Number) {
						editor.putFloat(key, (float) data.getDouble(key));
					} else if (data.getType(key) == ReadableType.Boolean) {
						editor.putBoolean(key, data.getBoolean(key));
					} else {
						Log.d(TAG, "Can't store " + data.getType(key) + " is unsupported");
					}
				}
				editor.commit();
			}
		} catch(Exception e){
			e.printStackTrace();
		}
	}

	private WritableMap getMultiple(List<Object> keys){
		WritableMap map = Arguments.createMap();
		try {
			Map<String, ?> all = getAll();
			for (Map.Entry<String, ?> entry : all.entrySet()) {
				if (keys == null || keys.contains(entry.getKey())) {
					if (entry.getValue() instanceof String) {
						map.putString(entry.getKey(), entry.getValue().toString());
					} else if (entry.getValue() instanceof Number) {
						//we need to convert number as string and then parse as double to avoid Javascript precision issue
						map.putDouble(entry.getKey(), Double.parseDouble(entry.getValue().toString()));
					} else if (entry.getValue() instanceof Boolean) {
						map.putBoolean(entry.getKey(), (boolean) entry.getValue());
					} else {
						Log.d(TAG, "Can't add " + entry.getValue() + ", its type is unsupported");
					}
				}
			}
		} catch (Exception e){
			e.printStackTrace();
		}
		return map;
	}

	private void clearMultiple(List<Object> keys){
		try {
			Map<String, ?> all = getAll();
			SharedPreferences.Editor editor = getEditor();
			for (Map.Entry<String, ?> entry : all.entrySet()) {
				if (keys == null || keys.contains(entry.getKey())) {
					editor.remove(entry.getKey());
				}
			}
			editor.commit();
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	private SharedPreferences getPreferences() {
		String name = this.preferencesName;
		if (name == null || name.length() == 0) {
			name = this.reactContext.getPackageName() + "_preferences";
		}
		return getReactApplicationContext().getSharedPreferences(name, Context.MODE_PRIVATE);
	}

	private Map<String, ?> getAll(){
		return getPreferences().getAll();
	}

	private SharedPreferences.Editor getEditor() {
		return getPreferences().edit();
	}
}
